<?php

namespace AutomateWoo\Async_Events;

defined( 'ABSPATH' ) || exit;

/**
 * @class AppointmentStatusChanged
 *
 * @since 4.18.1
 */
class AppointmentStatusChanged extends Abstract_Async_Event {

	const NAME = 'appointment_status_changed';

	/**
	 * Init the event.
	 */
	public function init() {
		add_action( 'woocommerce_appointment_status_changed', [ $this, 'schedule_event' ], 30, 3 );
	}

	/**
	 * Schedule appointments status change event for consumption by triggers.
	 *
	 * Doesn't dispatch for 'was-in-cart' status changes because this status isn't a real appointment status and essentially
	 * functions as a 'trash' status. The was in cart is used when a appointment cart item is removed from the cart.
	 *
	 * @param string $from       Previous status.
	 * @param string $to         New (current) status.
	 * @param int    $appointment_id Appointment id.
	 */
	public function schedule_event( string $from, string $to, int $appointment_id ) {
		$was_in_cart = 'was-in-cart';
		if ( $to === $was_in_cart || $from === $was_in_cart ) {
			// Don't dispatch an event for 'was-in-cart' status changes
			return;
		}

		$this->create_async_event(
			[
				$appointment_id,
				$from,
				$to,
			]
		);
	}
}
